@@ -23,14 +23,19 @@ except (SystemError, ImportError):
     from util import logging_setup, check_positive, check_positive_between_zero_and_one
     from hardrules import Hardrules
 
+# Remove fasttext warning
+fasttext.FastText.eprint = lambda x: None
+
 __author__ = "Sergio Ortiz Rojas"
 __version__ = "Version 1.0 # 24/05/2021 # Separate hardrules package from Bicleaner # Jaume Zaragoza"
 __version__ = "Version 1.1 # 26/05/2021 # Load lm only when necessary # Jaume Zaragoza"
 __version__ = "Version 1.3 # 05/07/2021 # FastSpell, bad encoding Ä, check identical with alphabetic and discard empty sides # Jaume Zaragoza"
 __version__ = "Version 1.3.1 # 12/07/2021 # Fix PyPi release # Jaume Zaragoza"
 __version__ = "Version 2.0 # 01/03/2022 # Parametrized rules and run all mode # Jaume Zaragoza"
-__version__= "Version 2.2 # 11/0/2022 # Added inconsistencies rules # Marta Bañón"
-__version__= "Version 2.3 # 26/07/2022 # Automated KenLM build # Jaume Zaragoza"
+__version__ = "Version 2.2 # 11/0/2022 # Added inconsistencies rules # Marta Bañón"
+__version__ = "Version 2.3 # 26/07/2022 # Automated KenLM build # Jaume Zaragoza"
+__version__ = "Version 2.4 # 16/11/2022 # Minor fixes and updates # Jaume Zaragoza"
+__version__ = "Version 2.5 # 23/11/2022 # Jaume Zaragoza"
 
 
 logging_level = 0
@@ -96,7 +101,7 @@ def initialization():
     if not args.disable_lang_ident:
         # Load a FastSpell objet to trigger download of fasttext langid
         # before running hardrules
-        fsobj = FastSpell.FastSpell("en", mode="cons")
+        fsobj = FastSpell("en", mode="aggr")
 
     #Try loading metadata for LM filtering and porn removal
     if not (args.disable_lm_filter and args.disable_porn_removal) and args.metadata != None:
@@ -234,7 +239,7 @@ def worker_process(i, jobs_queue, output_queue, args):
                         wrong_tu_results = hardrules.wrong_tu(left, right)
                     else:
                         logging.error("scol ({}) or tcol ({}) indexes above column number ({})".format(args.scol, args.tcol, len(parts)))
-                        wrong_tu_results = "c_wrong_cols"
+                        wrong_tu_results = ["c_wrong_cols"]
 
                     # Print input sentences when scoring_only is disabled
                     if not args.score_only:
