@@ -16,8 +16,6 @@ from collections import OrderedDict
 from inspect import getmembers, signature
 from copy import deepcopy
 
-
-
 try:
     from .lm import load_lm_filter
     from .tokenizer import Tokenizer
@@ -30,20 +28,22 @@ tbl_non_alpha = str.maketrans('', '', ''.join(tbl_non_alpha))
 regex_blank = regex.compile("[ \u00A0]")
 regex_alpha = regex.compile("[[:alpha:]]")
 regex_numbers = regex.compile("[[:digit:]]")
-regex_url = regex.compile('((?:https?://|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}/)(?:[^\s()<>]|\((:?[^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:\'".,<>?\xab\xbb\u201c\u201d\u2018\u2019]))')
+#regex_url = regex.compile('((?:https?://|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}/)(?:[^\s()<>]|\((:?[^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:\'".,<>?\xab\xbb\u201c\u201d\u2018\u2019]))')
+regex_url = regex.compile("(http(s)?:\/\/.)?(www\.)?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)")
 #regex_breadcrumbs = regex.compile("([ ][-/»][ ]|[|<>→←]|[ ][:][:][ ])")
 regex_breadcrumbs1 = regex.compile("([ ][-/][ ]|[<>*]|[ ][:][ ])")
 regex_breadcrumbs2 = regex.compile("([ ][»][ ]|[|→←•·¬])")
 regex_unicode_noise = regex.compile("[\x80-\xFF]{3,}")
 regex_spaces_noise = regex.compile("([ ]\D){4,}[ ]")
-regex_paren = regex.compile("[][(){}]")
+#regex_paren = regex.compile("[][(){}]")
+regex_paren = r"\[|\]|\(|\)|{|}|⟨|⟩"
 regex_unwanted = regex.compile("[+*]")
 regex_inconditional = regex.compile("=\"")
-regex_escaped_unicode = regex.compile("[\\\\]u[0-9a-fA-F]{3,}")
+regex_escaped_unicode = regex.compile("[\\\\][xu][0-9a-fA-F]{2,}") #matches stuff like \\u245 and \\xc3
 #regex_glued_words = regex.compile("\b[[:alpha:]]*[[:lower:]][[:upper:]][[:alpha:]]*)
 regex_glued_words = regex.compile("([[:alpha:]]*[[:upper:]]{1}[[:lower:]]+){3}")
-regex_repeated_words = regex.compile(r"\b(.+)\\1\b")
-regex_repeated_without_words = regex.compile(r"(.+)\\1")
+regex_repeated_words = regex.compile(r"\b(.+)\1\b")
+regex_repeated_without_words = regex.compile(r"(.+)\1")
 
 safe_noise_detection_langs = {"en", "es", "fr", "pl", "de", "it", "pt", "nl", "cs", "ro", "fi", "lv", "et", "bg", "hr", "da", "hu", "ga", "eu", "gl", "sl", "sv", "mt", "sk", "is", "lt", "nb", "nn", "no"}
 
@@ -64,7 +64,7 @@ class Hardrules():
     rule_pipeline['no_literals'] = ["Re:","{{", "%s", "}}", "+++", "***", '=\"']
     rule_pipeline['no_only_symbols'] = True
     rule_pipeline['no_only_numbers'] = True
-    rule_pipeline['no_urls'] = True
+    rule_pipeline['no_urls'] = False
     rule_pipeline['no_breadcrumbs'] = True
     rule_pipeline['no_glued_words'] = True
     rule_pipeline['no_repeated_words'] = True
@@ -111,8 +111,8 @@ class Hardrules():
 
         # Load FastSpell
         if not args.disable_lang_ident:
-            self.fastspell_src = FastSpell.FastSpell(args.source_lang, mode="aggr")
-            self.fastspell_trg = FastSpell.FastSpell(args.target_lang, mode="aggr")
+            self.fastspell_src = FastSpell(args.source_lang, mode="aggr")
+            self.fastspell_trg = FastSpell(args.target_lang, mode="aggr")
         else:
             self.fastspell_src = None
             self.fastspell_trg = None
@@ -156,7 +156,6 @@ class Hardrules():
         for rule_name in self.config.keys():
             if not self.config[rule_name]:
                 continue
-            logging.debug(f"Rule: {rule_name}")
 
             # Obtain function to be applied
             rule_func = self.rules['c_' + rule_name]
@@ -167,8 +166,7 @@ class Hardrules():
                 # Iterate over left and right side
                 for sidename, side in {'left': left, 'right': right}.items():
                     keep = rule_func(side, sidename)
-                    logging.debug(f"Side: {sidename}")
-                    logging.debug(f"Keep: {keep}")
+                    logging.debug(f"Rule '{rule_name}({sidename})': {not keep}")
                     if not keep and self.run_all_rules:
                         # Especial case for empty rule to avoid crashes in other rules
                         if rule_name == 'no_empty':
@@ -185,7 +183,7 @@ class Hardrules():
             else:
                 # Apply rule to both sides
                 keep = rule_func(left, right)
-                logging.debug(f"Keep: {keep}")
+                logging.debug(f"Rule '{rule_name}(left,right)': {not keep}")
                 if not keep and self.run_all_rules:
                     discards.append(f"{rule_name}(left,right)")
                 elif not keep:
@@ -271,7 +269,9 @@ class Hardrules():
         return len(regex_numbers.findall(sentence)) / len(sentence) < threshold
 
     def c_no_urls(self, sentence, side):
-        return sum([len("".join(i)) for i in regex_url.findall(sentence)]) < 15
+        #return sum([len("".join(i)) for i in regex_url.findall(sentence)]) < 15
+        return  len(regex_url.findall(sentence) )== 0
+
 
     def c_no_breadcrumbs(self, sentence, side):
         return len(regex_breadcrumbs1.findall(sentence)) < 3 \
@@ -283,8 +283,34 @@ class Hardrules():
     def c_no_space_noise(self, sentence, side):
         return len(regex_spaces_noise.findall(sentence)) == 0
 
-    def c_no_paren(self, sentence, side):
-        return len(regex_paren.findall(sentence)) < 10
+    def c_no_paren(self,left, right):
+        if len(re.findall(regex_paren, left)) or len(re.findall(regex_paren, right)): #there are parentheses
+            l_char_count = {i: left.count(i) for i in set(left)}
+            r_char_count = {i: right.count(i) for i in set(right)}
+            if (((l_char_count.get("[") or 0) + (l_char_count.get("]") or 0)) > 6 ) or (l_char_count.get("[") or 0) != (l_char_count.get("]") or 0): #max 6 [ or ], having the same [ and ] 
+                return False
+            if (((r_char_count.get("[") or 0) + (r_char_count.get("]") or 0)) > 6 ) or (r_char_count.get("[") or 0) != (r_char_count.get("]") or 0): #max 6 [ or ], having the same [ and ] 
+                return False                
+            if (((l_char_count.get("{") or 0) + (l_char_count.get("}") or 0)) > 6 ) or (l_char_count.get("{") or 0) != (l_char_count.get("}") or 0): #max than 6 { or }, having the same { and }
+                return False
+            if (((r_char_count.get("{") or 0) + (r_char_count.get("}") or 0)) > 6 ) or (r_char_count.get("{") or 0) != (r_char_count.get("}") or 0): #max than 6 { or }, having the same { and }
+                return False                
+            if (((l_char_count.get("⟨") or 0) + (l_char_count.get("⟩") or 0)) > 6 ) or (l_char_count.get("⟨") or 0) != (l_char_count.get("⟩") or 0): #max than 6 ⟨ or ⟩, having the same ⟨ and ⟩            
+                return False
+            if (((r_char_count.get("⟨") or 0) + (r_char_count.get("⟩") or 0)) > 6 ) or (r_char_count.get("⟨") or 0) != (r_char_count.get("⟩") or 0): #max than 6 ⟨ or ⟩, having the same ⟨ and ⟩            
+                return False                
+            l_opening_paren = l_char_count.get("(") or 0
+            r_opening_paren = r_char_count.get("(") or 0
+            l_closing_paren = l_char_count.get(")") or 0
+            r_closing_paren = r_char_count.get(")") or 0
+            if (l_opening_paren == l_closing_paren) and (r_opening_paren == r_closing_paren): #any amount of  () is allowed, as long as there are the same amount of  ( and )
+                return True
+            elif l_closing_paren == r_closing_paren:     #if not the same amount, the closing ) must match in source and target, to allow  sentences as a)..., b)..., etc
+                return True
+            else:
+                return False                
+        return True    
+
 
     def c_no_literals(self, sentence, side):
         return not any(l in sentence for l in self.config["no_literals"])
